@@ -1,5 +1,9 @@
 from . import _version
 from .animator import GifAnimator, Mp4Animator  # noqa
-from .preprocessor import Preprocessor, MatplotlibPreprocessor  # noqa
+from .preprocessor import (  # noqa
+    Preprocessor,
+    MatplotlibPreprocessor,
+    HoloViewsPreprocessor,
+)
 
 __version__ = _version.get_versions()["version"]
